package com.pwt.web.mapper;

import java.util.List;

import com.pwt.web.domain.PwtBattery;
import com.pwt.web.domain.PwtVehicle;
import org.apache.ibatis.annotations.Param;

/**
 * 车辆管理Mapper接口
 * 
 * @author linyun
 * @date 2022-11-17
 */
public interface PwtVehicleMapper 
{
    /**
     * 查询车辆管理
     * 
     * @param vehicleId 车辆管理ID
     * @return 车辆管理
     */
    public PwtVehicle selectPwtVehicleById(Long vehicleId);

    /**
     * 查询车辆管理列表
     * 
     * @param pwtVehicle 车辆管理
     * @return 车辆管理集合
     */
    public List<PwtVehicle> selectPwtVehicleList(PwtVehicle pwtVehicle);

    /**
     * 新增车辆管理
     * 
     * @param pwtVehicle 车辆管理
     * @return 结果
     */
    public int insertPwtVehicle(PwtVehicle pwtVehicle);

    /**
     * 修改车辆管理
     * 
     * @param pwtVehicle 车辆管理
     * @return 结果
     */
    public int updatePwtVehicle(PwtVehicle pwtVehicle);

    /**
     * 删除车辆管理
     * 
     * @param vehicleId 车辆管理ID
     * @return 结果
     */
    public int deletePwtVehicleById(Long vehicleId);

    /**
     * 批量删除车辆管理
     * 
     * @param vehicleIds 需要删除的数据ID
     * @return 结果
     */
    public int deletePwtVehicleByIds(Long[] vehicleIds);

    /**
     * @Description: 查询车辆vin编码是否唯一
     * @Author: linyun
     * @Date: 2022/11/19 15:11
     * @Param:[vin 车辆vin编码]
     * @Return:int
     **/
    public int checkVehicleVinUnique(@Param("vin") String vin);

    /**
     * @Description: 查询车牌号是否唯一
     * @Author: linyun
     * @Date: 2022/11/19 15:11
     * @Param:[vehicleNo 车牌号]
     * @Return:int
     **/
    public int checkVehicleNoUnique(@Param("vehicleNo") String vehicleNo);

    /**
     * @Description: rfid唯一性校验
     * @Author: linyun
     * @Date: 2022/11/19 16:52
     * @Param:[rfid]
     * @Return:int
     **/
    public int checkVehicleRfidUnique(@Param("rfid") String rfid);

    /**
     * @Description: 根据vin编码查询
     * @Author: linyun
     * @Date: 2022/11/19 15:20
     * @Param:[vin vin编码]
     * @Return:PwtVehicle
     **/
    public PwtVehicle getByVinCode(@Param("vin") String vin);

    /**
     * @Description: 根据车牌号查询
     * @Author: linyun
     * @Date: 2022/11/19 15:20
     * @Param:[vehicleNo 车牌号]
     * @Return:PwtVehicle
     **/
    public PwtVehicle getByVehicleNo(@Param("vehicleNo") String vehicleNo);

    /**
     * @Description: 根据rfid查询
     * @Author: linyun
     * @Date: 2022/11/19 17:03
     * @Param:[rfid 车辆rfid]
     * @Return:com.pwt.web.domain.PwtVehicle
     **/
    public PwtVehicle getByRfid(@Param("rfid") String rfid);

    public int getCount();
}
