package com.pwt.web.service;

import java.util.List;
import com.pwt.web.domain.PwtVehicle;

/**
 * 车辆管理Service接口
 * 
 * @author linyun
 * @date 2022-11-17
 */
public interface IPwtVehicleService 
{
    /**
     * 查询车辆管理
     * 
     * @param vehicleId 车辆管理ID
     * @return 车辆管理
     */
    public PwtVehicle selectPwtVehicleById(Long vehicleId);

    /**
     * 查询车辆管理列表
     * 
     * @param pwtVehicle 车辆管理
     * @return 车辆管理集合
     */
    public List<PwtVehicle> selectPwtVehicleList(PwtVehicle pwtVehicle);

    /**
     * 新增车辆管理
     * 
     * @param pwtVehicle 车辆管理
     * @return 结果
     */
    public int insertPwtVehicle(PwtVehicle pwtVehicle);

    /**
     * 修改车辆管理
     * 
     * @param pwtVehicle 车辆管理
     * @return 结果
     */
    public int updatePwtVehicle(PwtVehicle pwtVehicle);

    /**
     * 批量删除车辆管理
     * 
     * @param vehicleIds 需要删除的车辆管理ID
     * @return 结果
     */
    public int deletePwtVehicleByIds(Long[] vehicleIds);

    /**
     * 删除车辆管理信息
     * 
     * @param vehicleId 车辆管理ID
     * @return 结果
     */
    public int deletePwtVehicleById(Long vehicleId);

    /**
     * @Description: 导入车辆数据
     * @Author: linyun
     * @Date: 2022/11/19 15:05
     * @Param:[list 导入的数据]
     * @Return:java.lang.String
     **/
    public String importData(List<PwtVehicle> list);

    /**
     * 根据rfid查询VIN码
     * @param rfid
     * @return VIN码
     */
    public PwtVehicle getVinByRfid(String rfid);

    /**
     * 验证vin是否存在
     * @param vin vin码
     * @return true:存在 false:不存在
     */
    public boolean checkVinExist(String vin);

    public int getCount();
}
